#!/usr/bin/env python
# -*- coding: utf-8  -*-

##    Copyright (C) 2008 Roberto Zanon
##    This program is free software: you can redistribute it and/or modify
##    it under the terms of the GNU General Public License as published by
##    the Free Software Foundation, either version 3 of the License, or
##    (at your option) any later version.
##
##    This program is distributed in the hope that it will be useful,
##    but WITHOUT ANY WARRANTY; without even the implied warranty of
##    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##    GNU General Public License for more details.
##
##    You should have received a copy of the GNU General Public License
##    along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys, subprocess, os
import xml.etree.ElementTree as ET
from PyQt4 import QtGui,QtCore

# main assigns an instance of Config class.
global config


class Config():
    """Contains configuration variable: iconDirectory, systemTrayIcon.
    """

    def __init__(self, xConfigElement):
        """xConfigElement is the "config" xml element .
        """
        self._xConfigElement = xConfigElement
    
    def __getattr__(self, name):
        if name == 'iconDirectory':
            iconDir = self._xConfigElement.findtext('./iconDirectory')
            if not os.path.isabs(iconDir):
                # Append relative path iconDir to the path in which is kSimpleRun.py
                kSimpleRun_dir = os.getcwdu() + os.path.sep
                path, file = os.path.split(sys.argv[0])
                kSimpleRun_dir += os.path.sep + path
                iconDir = kSimpleRun_dir + os.path.sep + iconDir
            if not os.path.isdir(iconDir):
                sys.stderr.write('"%s" doesn\'t exist. Check "iconDir" element in config.xml\n'
                            % iconDir)
            return iconDir + os.path.sep
        elif name == 'systemTrayIcon':
            return self._xConfigElement.findtext('./systemTrayIcon')
        elif name == 'exitIcon':
            return self._xConfigElement.findtext('./exitIcon')
        raise AttributeError()


class Executable():
    """It makes simpler to connect a qt signal to a command to execute in a shell.
    """
    def __init__(self, command):
        self.command = command

    def __call__(self):
        subprocess.Popen([self.command], shell=True)


class Operation():
    """It rapresesents an operation composed by: command, message and icon.
    Command, message and icon can be accessed ad object variable.
    """

    def __init__(self, xActionElement):
        """xActionElement is an xml element with attribute "message" and "icon",
        and contains child element "command". "Icon" isn't mandatory.
        """
        self._actionElement = xActionElement
    
    def __getattr__(self, name):
        if name == 'icon':
            iconName = self._actionElement.get('icon')
            if iconName:
                return QtGui.QIcon(config.iconDirectory + iconName)
            else:
                return None
        elif name == 'message':
            return self._actionElement.get('message')
        elif name == 'command':
            command = self._actionElement.findtext('./command')
            return Executable(command)
        raise AttributeError()


class ComplexMenu(QtGui.QMenu):
    """It's a inner menu; a command with an argument to choose.
    """
    def __init__(self, xActionElement, parent=None):
        """xActionElement is an "actionComplex" xml element.
        """
        QtGui.QMenu.__init__(self, parent)
        operation = Operation(xActionElement)
        self.setTitle(operation.message)
        if operation.icon:
            self.setIcon(operation.icon)
        commandElement = xActionElement.find('./command')
        command = {}
        command['Left'] = commandElement.text
        command['Right'] = commandElement[0].tail
        if not command['Right']: command['Right'] = ''
        xOptionElements = xActionElement.findall('./argOptions/option')
        for xOptionElement in xOptionElements:
            optionOperation = Operation(xOptionElement)
            command['Param'] = xOptionElement.text
            addedAction = self.addAction(optionOperation.message,
                        Executable(command['Left'] + command['Param'] + command['Right']))
            if optionOperation.icon:
                addedAction.setIcon(optionOperation.icon)

class MainMenu(QtGui.QMenu):
    """MainMenu that contains actions (commands with constant arguments)
    and inner menu (commands with argument to choose).
    """
    def __init__(self, xActionElements, parent=None):
        QtGui.QMenu.__init__(self, parent)
        
        #add action and inner menu.
        for xActionElement in xActionElements:
            if xActionElement.tag == 'action':
                operation = Operation(xActionElement)
                self.addOperation(operation)
            elif xActionElement.tag == 'actionComplex':
                subMenu = ComplexMenu(xActionElement, self)
                self.addMenu(subMenu)
        
        # add separator and exit action
        self.addSeparator()
        exitAction = self.addAction(QtGui.QIcon(config.iconDirectory + config.exitIcon),"Exit")
        self.connect(exitAction, QtCore.SIGNAL("triggered()"), QtGui.qApp, QtCore.SLOT("quit()"))

    def addOperation(self, operation):
        """Add an action corresponding to "operation" (an instance Operation).
        """
        action = self.addAction(operation.message, operation.command)
        if operation.icon:
            action.setIcon(operation.icon)


def main():
    # Init configuration
    configTree = ET.parse(os.path.join(os.path.expanduser("~"), ".kSimpleRun.xml"))
    global config
    config = Config(configTree.find("/config"))
    # Init gui.
    kSimpleRun = QtGui.QApplication(sys.argv)
    sysTrayIcon = QtGui.QSystemTrayIcon()
    icon = QtGui.QIcon(config.iconDirectory + config.systemTrayIcon)
    sysTrayIcon.setIcon(icon)
    ActionsMenu = MainMenu(configTree.findall('/actions/*'))
    sysTrayIcon.setContextMenu(ActionsMenu)
    # Run gui
    sysTrayIcon.show()
    sys.exit(kSimpleRun.exec_())

if __name__ == '__main__':
    main()
